from random import *

class Primzahlgenerator(object):
    """
    Bauplan zur Erzeugung von Objekten,
    die Primzahlen nach dem
    Miller-Rabin-Verfahren erzeugen
    """

    def __init__(self):
        """
        erzeugt ein Objekt der Klasse Primzahlgenerator mit
        vorgegebener Anzahl von Durchgängen beim Miller-Rabin-Test
        """
        self.anzahlWiederholungen = 20
        seed() # initialisiert den Zufallsgenerator

    def setAnzahlWiederholungen(self, i):
        """
        setzt die Anzahl von Durchgängen beim Miller-Rabin-Test
        """
        self.anzahlWiederholungen = i

    def getAnzahlWiederholungen(self):
        """
        gibt die Anzahl von Durchgängen beim Miller-Rabin-Test zurück
        """
        return self.anzahlWiederholungen

    def millerRabinPass(self, a, n):
        """
        Hilfsfunktion zur Durchführung des Miller-Rabin-Tests
        """
        d = n - 1
        s = 0
        while d % 2 == 0:
            d = d >> 1
            s = s + 1
        a_to_power = pow(a, d, n)
        if a_to_power == 1:
            return True
        for i in range(s-1):
            if a_to_power == n-1:
                return True
            a_to_power = (a_to_power * a_to_power) % n
        return a_to_power == n - 1

    def istWahrscheinlichPrimzahl(self, n):
        """
        führt den Miller-Rabin-Test durch
        """
        if n <= 1:
            wahrscheinlichPrim = False
        else:
            wahrscheinlichPrim = True
            for i in range(self.anzahlWiederholungen):
                a = randint(1, n-1)
                if not self.millerRabinPass(a, n):
                    wahrscheinlichPrim = False
        return wahrscheinlichPrim

    def naechstePrimzahl(self, m):
        p = m
        prim = False
        while not prim:
            if self.istWahrscheinlichPrimzahl(p):
                prim = True
            else:
                p = p+1
        return p

    def erzeugtePrimzahl(self, m, n):
        p = None
        return p


